using System;
using System.Runtime.InteropServices;
using UnityEngine;
using LitJson;
using System.Collections.Generic;
using TapTapMiniGame;
using UnityEngine.Scripting;

public class TapManagerHandler : MonoBehaviour
{
    // 统一字典和通用回调类型
    public class CommonShareboardOption
    {
        public Action<TapCallbackResult> success;
        public Action<TapCallbackResult> fail;
        public Action<TapCallbackResult> complete;
    }

    private static Dictionary<string, CommonShareboardOption> shareboardCallbackDict;

    private static TapManagerHandler _instance = null;

    public static TapManagerHandler Instance
    {
        get
        {
            //IL_003c: Unknown result type (might be due to invalid IL or missing references)
            if ((UnityEngine.Object)(object)_instance == (UnityEngine.Object)null)
            {
                if (!Application.isPlaying)
                {
                    Debug.LogError((object)"不支持在非播放模式下调用Tap接口");
                    return null;
                }

                _instance = new GameObject(typeof(TapManagerHandler).Name).AddComponent<TapManagerHandler>();
                UnityEngine.Object.DontDestroyOnLoad((UnityEngine.Object)(object)((Component)_instance).gameObject);
            }

            return _instance;
        }
    }

    public static string GetCallbackId<T>(Dictionary<string, T> dict)
    {
        int num = dict.Count;
        string text = ((float)num + UnityEngine.Random.value).ToString();
        while (dict.ContainsKey(text))
        {
            num++;
            text = ((float)num + UnityEngine.Random.value).ToString();
        }

        return text;
    }


    [DllImport("__Internal", EntryPoint = "formatJsonStr")]
    [Preserve]
    private static extern void FormatJsonStr();

    #region SetShareBoard & ShowShareboard

    // 修改后的 ShareboardCallback 函数
    public void ShareboardCallback(string msg)
    {
        if (string.IsNullOrEmpty(msg))
            return;
        TapJSCallback callback = JsonUtility.FromJson<TapJSCallback>(msg);
        string callbackId = callback.callbackId;
        if (shareboardCallbackDict == null || !shareboardCallbackDict.ContainsKey(callbackId))
            return;
        CommonShareboardOption option = shareboardCallbackDict[callbackId];
        TapCallbackResult result = JsonUtility.FromJson<TapCallbackResult>(callback.res);

        switch (callback.type)
        {
            case "complete":
                option.complete?.Invoke(result);
                option.complete = null;
                shareboardCallbackDict.Remove(callbackId);
                break;
            case "fail":
                option.fail?.Invoke(result);
                option.fail = null;
                option.success = null;
                break;
            case "success":
                option.success?.Invoke(result);
                option.fail = null;
                option.success = null;
                break;
        }
    }

    [DllImport("__Internal", EntryPoint = "Tap_SetShareboardHidden")]
    private static extern void Tap_SetShareboardHidden(string option, string callbackId);

    public void SetShareboardHidden(SetShareboardHiddenOption option)
    {
        if (shareboardCallbackDict == null)
        {
            shareboardCallbackDict = new Dictionary<string, CommonShareboardOption>();
        }
        string callbackId = GetCallbackId(shareboardCallbackDict);
        // 统一转换为 CommonShareboardOption
        CommonShareboardOption value = new CommonShareboardOption
        {
            success = option.success,
            fail = option.fail,
            complete = option.complete
        };
        shareboardCallbackDict.Add(callbackId, value);
        Action<TapCallbackResult> success = option.success;
        Action<TapCallbackResult> fail = option.fail;
        Action<TapCallbackResult> complete = option.complete;
        option.success = null;
        option.fail = null;
        option.complete = null;
        string text = JsonMapper.ToJson(option);
        option.success = success;
        option.fail = fail;
        option.complete = complete;

        Tap_SetShareboardHidden(text, callbackId);
    }

    [DllImport("__Internal", EntryPoint = "Tap_ShowShareboard")]
    private static extern void Tap_ShowShareboard(string option, string callbackId);

    public void ShowShareboard(ShowShareboardOption option)
    {
        if (shareboardCallbackDict == null)
        {
            shareboardCallbackDict = new Dictionary<string, CommonShareboardOption>();
        }
        string callbackId = GetCallbackId(shareboardCallbackDict);
        CommonShareboardOption value = new CommonShareboardOption
        {
            success = option.success,
            fail = option.fail,
            complete = option.complete
        };
        shareboardCallbackDict.Add(callbackId, value);
        Action<TapCallbackResult> success = option.success;
        Action<TapCallbackResult> fail = option.fail;
        Action<TapCallbackResult> complete = option.complete;
        option.success = null;
        option.fail = null;
        option.complete = null;
        string text = JsonMapper.ToJson(option);
        option.success = success;
        option.fail = fail;
        option.complete = complete;

        Tap_ShowShareboard(text, callbackId);
    }

    #endregion

    #region OnShareMessage and OffShareMessage

    // 监听器类型保持不变: 第一个参数为 resolve 函数，第二个为 channel 字符串
    private static Action<Action<ShareMessageParam>, string> onShareMessageListener;

    [DllImport("__Internal", EntryPoint = "Tap_OnShareMessage")]
    private static extern void Tap_OnShareMessage();

    // OnShareMessage 不再合并代理，直接赋值
    public void OnShareMessage(Action<Action<ShareMessageParam>, string> action)
    {
        onShareMessageListener = action;
        Tap_OnShareMessage();
    }

    // 修改后的 OnShareMessageCallback 函数
    public void OnShareMessageCallback(string msg)
    {
        if (string.IsNullOrEmpty(msg))
        {
            return;
        }
        // 直接将 msg 当作 channel 字符串传递
        string channel = msg;
        onShareMessageListener?.Invoke(delegate (ShareMessageParam param)
        {
            if (param == null)
            {
                param = new ShareMessageParam();
            }
            // 使用 shareboardCallbackDict 替代缓存逻辑
            if (shareboardCallbackDict == null)
            {
                shareboardCallbackDict = new Dictionary<string, CommonShareboardOption>();
            }
            string callbackId = GetCallbackId(shareboardCallbackDict);
            CommonShareboardOption value = new CommonShareboardOption
            {
                success = param.success,
                fail = param.fail,
                complete = param.complete
            };
            shareboardCallbackDict.Add(callbackId, value);
            // 序列化前，临时清除回调，防止回调函数被序列化
            Action<TapCallbackResult> success = param.success;
            Action<TapCallbackResult> fail = param.fail;
            Action<TapCallbackResult> complete = param.complete;
            param.success = null;
            param.fail = null;
            param.complete = null;
            string text = JsonMapper.ToJson(param);
            param.success = success;
            param.fail = fail;
            param.complete = complete;
            Tap_OnShareMessageResolve(text, callbackId);
        }, channel);
    }

    [DllImport("__Internal", EntryPoint = "Tap_OnShareMessageResolve")]
    private static extern void Tap_OnShareMessageResolve(string param, string callbackId);

    [DllImport("__Internal", EntryPoint = "Tap_OffShareMessage")]
    private static extern void Tap_OffShareMessage();

    // 删除多播逻辑，直接清空代理即可
    public void OffShareMessage()
    {
        onShareMessageListener = null;
        Tap_OffShareMessage();
    }

    #endregion
}